%%%%%%%%%%%%%
% THR_HET.M %
%%%%%%%%%%%%%

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/


% This is a Matlab procedure.
% It computes a test for a threshold in linear regression
% allowing for heteroskedasticity.  The procedure takes the form

% [f_test,p_value] = thr_test(dat,yi,xi,qi,trim_per,rep)

% The inputs are:
% dat      = data matrix (nxk)
% yi       = index of dependent (y) variable, e.g. yi = 1
% xi       = indexes of independent (x) variables, e.g. xi = 2|3
% qi       = index of threshold (q) variable, e.g. qi = 4
% trim_per = percentage of sample to trim from ends, e.g. trim_per = .15
% rep      = number of bootstrap replications, e.g. rep = 1000

% Output:
% f_test   = Value of Maximal (Quandt) F-statistic
% p_value  = Bootstrap p-value

% Notes:
% (1)    Do not include a constant in the independent variables,
%        the program automatically adds an intercept to the regression.

% (2)    There are two bootstrap methods which the program can use.

%        The first method, obtained by setting _quick=1 at the beginning
%        of the procedure code, is the method presented in my paper
%        "Inference When a Nuisance Parameter is Not Identified Under
%        the Null Hypothesis" which simulates the asymptotic null
%        distribution.  A computational shortcut is also taken which speeds
%        computational time, at the cost of greater memory usage, so may
%        not be possible for large data sets.

%        The second method, obtained by setting quick_=2 at the beginning
%        of the procedure code, is a "fixed regressor bootstrap", which is
%        quite close.  The difference is that the bootstrap procedure calculates
%        the variance-covariance matrix in each bootstrap replication.  This
%        results in a better finite sample approximation.  The cost is greater
%        computation time.

%        The program is set by default to use the second method, which has better
%        sampling properties.  If computational time is a concern, switch to the
%        first (set _quick=1).  If an "out of workspace memory" message appears,
%       switch back to quick_=2.


% Example:
% If the nxk matrix "dat" contains the dependent variable in the first
% column, the independent variables in the second through tenth columns,
% and the threshold variable in the fifth:

% xi = [2;3;4;5;6;7;8;9;10];
% [f_test,p_value] = thr_test(dat,1,xi,5,.15,1000);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [ftest,pvalue]=thr_test(dat,yi,xi,qi,trim_per,rep);

% Control Parameters, May be Changed %

cr_=.95;      % This is the confidence level used to plot the
              %           critical value in the graph.  It is not used
              %           elsewhere in the analysis.  
graph_=1;     % Graph indicator
              %           Set graph_=0 to not view the graph of the likelihood
              %           Set graph_=1 to view the graph of the likelihood    
quick_=2;     % Indicator of method used for bootstrap
              %           Set quick_=1 for quick computation of asymptotic
              %           distribution.  This method is not a proper bootstrap
              %           and may result in excess rejections.  It also uses
              %           more memory.
              %           Set quick_=2 for a better bootstrap procedure, which
              %           also uses less memory, but is more time consuming   
           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n=length(dat(:,1));
q=dat(:,qi);

% Sorted index of q %
qs=0;
for i=1:length(q)
    for i_=1:length(q)
        temp=0;
        for j=1:length(q)
            if (q(i_)>q(j))|(i_>=j&q(i_)==q(j))
                temp=temp+1;
            end;
        end;
        if temp==i
            qs=[qs;i_];
        end;
    end;
end;
qs=qs(2:length(qs));
q=q(qs);

y=dat(qs,yi);
x=[ones(n,1),dat(qs,xi)];
k=length(x(1,:));
qs=unique(q);
qn=length(qs(:,1));
qq=zeros(qn,1);
r=1;
while r<=qn
    qq(r)=sum(q==qs(r));
    r=r+1;
end;
cqq=cumsum(qq)';
sq=(cqq>=floor(n*trim_per)).*(cqq<=floor(n*(1-trim_per)));

temp_qs=0;temp_cqq=0;ind=0;
for i=1:length(sq)
    if sq(i)==1
        if ind==0
            temp_qs=qs(i);
            temp_cqq=cqq(i);
            ind=1;
        else
            temp_qs=[temp_qs;qs(i)];
            temp_cqq=[temp_cqq;cqq(i)];
        end;
    end;
end;
qs=temp_qs;
cqq=temp_cqq;
clear temp_qs;
clear temp_cqq;
qn=length(qs);

mi=inv(x'*x);
e=y-x*mi*(x'*y);
ee=e'*e;
xe=x.*(e*ones(1,length(x(1,:))));
vi=xe'*xe;
cxe=cumsum(xe);
sn=zeros(qn,1);

if quick_==1
    mmistore=zeros(k*(k+1)/2,qn);
    cqqb=1;
    mm=zeros(k,k);
    vv=zeros(k,k);
    r=1;
    while r<=qn
        cqqr=cqq(r);
        mm=mm+x(cqqb:cqqr,:)'*x(cqqb:cqqr,:);
        vv=vv+xe(cqqb:cqqr,:)'*xe(cqqb:cqqr,:);
        sume=cxe(cqqr,:)';
        mmi=inv(vv - mm*mi*vv - vv*mi*mm + mm*mi*vi*mi*mm);
        sn(r)=sume'*mmi*sume;
        cqqb=cqqr+1;
        indmmi=1;
        for i=1:length(mmi(:,1))
            ii=1;
            while ii<=i
                mmistore(indmmi,r)=mmi(i,ii);
                indmmi=indmmi+1;
                ii=ii+1;
            end;
        end;
        clear indmmi;
        clear ii;
        r=r+1;
    end;
    for i=1:length(sn) %maxindc%   
        if sn(i)==max(sn)
           si=i;
           break;
        end;
    end;
    qmax=qs(si);
    lr=sn;
    ftest=sn(si);
    fboot=zeros(rep,1);
    j=1;
    while j<=rep
        y=randn(n,1).*e;
        xe=x.*((y-x*mi*(x'*y))*ones(1,length(x(1,:))));
        cxe=cumsum(xe);
        sn=zeros(qn,1);
        r=1;
        while r<=qn
            ind=1; % xpnd % 
            for ix=1:sqrt(2*length(mmistore(:,r)))
               for iy=1:ix
                   mmi(ix,iy)=mmistore(ind,r);
                   mmi(iy,ix)=mmistore(ind,r);
                   ind=ind+1;
               end;
           end;
           clear ix;
           clear iy;
           sume=cxe(cqq(r),:)';
           sn(r)=sume'*mmi*sume;
           r=r+1;
        end;
        fboot(j)=max(sn)';
        j=j+1;
    end;
elseif quick_==2
    cqqb=1;
    mm=zeros(k,k);
    vv=zeros(k,k);
    r=1;
    while r<=qn
        cqqr=cqq(r);
        mm=mm+x(cqqb:cqqr,:)'*x(cqqb:cqqr,:);
        vv=vv+xe(cqqb:cqqr,:)'*xe(cqqb:cqqr,:);
        sume=cxe(cqqr,:)';
        mmi=vv - mm*mi*vv - vv*mi*mm + mm*mi*vi*mi*mm;
        warning off;
        lastwarn(' ');
        temp_sum=inv(mmi)*sume;
        [mw,idw] = lastwarn;
        lastwarn(' ');
        warning on;
        if mw==' ';
            sm=inv(mmi)*sume;
            sn(r)=sume'*sm;
        end;
        cqqb=cqqr+1;
        r=r+1;
    end;
    for i=1:length(sn) %maxindc%
        if sn(i)==max(sn)
            si=i;
            break;
        end;
    end;
    qmax=qs(si);
    lr=sn;
    ftest=sn(si);
    fboot=zeros(rep,1);
    j=1;
    while j<=rep
        y=randn(n,1).*e;
        xe=x.*((y-x*mi*(x'*y))*ones(1,length(x(1,:))));
        vi=xe'*xe;
        cxe=cumsum(xe);
        sn=zeros(qn,1);
        cqqb=1;
        mm=zeros(k,k);
        vv=zeros(k,k);
        r=1;
        while r<=qn
            cqqr=cqq(r);
            mm=mm+x(cqqb:cqqr,:)'*x(cqqb:cqqr,:);
            vv=vv+xe(cqqb:cqqr,:)'*xe(cqqb:cqqr,:);
            sume=cxe(cqqr,:)';
            mmi=vv - mm*mi*vv - vv*mi*mm + mm*mi*vi*mi*mm;
            warning off;
            lastwarn(' ');
            temp_sum=inv(mmi)*sume;
            [mw,idw] = lastwarn;
            lastwarn(' ');
            warning on;
            if mw==' ';
                sm=inv(mmi)*sume;
                sn(r)=sume'*sm;
            end;
            cqqb=cqqr+1;
            r=r+1;
        end;
        fboot(j)=max(sn)';
        j=j+1;
    end;
end;
fboot=sortrows(fboot,1);
pv=mean(fboot>=ftest)';
cr=fboot(round(rep*cr_));

% Output %
disp(' ');
disp(' ');
disp('Test of Null of No Threshold Against Alternative of Threshold')
disp('Allowing Heteroskedastic Errors (White Corrected)')
disp(' ');
fprintf('Number of Bootstrap Replications:  %u\n',rep);
fprintf('Trimming Percentage:               %f\n',trim_per);
disp(' ');
fprintf('Threshold Estimate:               %f\n',qmax);
fprintf('LM-test for no threshold:         %f\n',ftest);
fprintf('Bootstrap P-Value:                %f\n',pv);
disp(' ');
disp(' ');

% Graph %
if graph_==1;
    figure;
    clr=ones(qn,1)*cr;
    plot(qs,lr,qs,clr);
    ylabel('Fn(Gema)');
    xlabel('Gema');
    title('F Test For Threshold Reject Linearity if F Sequence Exceeds Critical Value');
    legend('LRN(Gema)',strcat(num2str(cr_*100),'% Critical'));
    grid on;
    
end;

%Latex%
ttex=fopen('out.tex','a+');
fprintf(ttex,'\\underline{\\large{Test of Null of No Threshold Against Alternative of Threshold}}\\\\\n');
fprintf(ttex,'\\small{Allowing Heteroskedastic Errors (White Corrected)}\\\\\n');
fprintf(ttex,'\\vspace{.1cm}\\\\\n');
fprintf(ttex,'\\textbf{Number of Bootstrap Replications:}%u\\\\\n',rep);
fprintf(ttex,'\\textbf{Trimming Percentage:}%f\\\\\n',trim_per);
fprintf(ttex,'\\vspace{.1cm}\\\\\n');
fprintf(ttex,'\\textbf{Threshold Estimate:}%f\\\\\n',qmax);
fprintf(ttex,'\\textbf{LM-test for no threshold:}%f\\\\\n',ftest);
fprintf(ttex,'\\textbf{Bootstrap P-Value:}%f\n',pv);
fprintf(ttex,'\\vspace{.1cm}\\\\\n');
fprintf(ttex,'\\vspace{.2cm}\\\\\n');
fclose(ttex);

